@@ -6,6 +6,7 @@ This module defines a versionless model for ``ROOT::RNTuple``.
 from __future__ import annotations
 
 import struct
+import sys
 from collections import defaultdict
 from itertools import accumulate
 
@@ -159,9 +160,7 @@ def _num_entries_for(in_ntuple, target_num_bytes, filter_name):
         if "column" in key and "union" not in key:
             key_nr = int(key.split("-")[1])
             for cluster in range(start_cluster_idx, stop_cluster_idx):
-                pages = in_ntuple.ntuple.page_list_envelopes.pagelinklist[cluster][
-                    key_nr
-                ].pages
+                pages = in_ntuple.ntuple.page_link_list[cluster][key_nr].pages
                 total_bytes += sum(page.locator.num_bytes for page in pages)
 
     total_entries = entry_stop
@@ -196,31 +195,30 @@ class Model_ROOT_3a3a_RNTuple(uproot.model.Model):
         keys = []
         field_records = self.field_records
         for i, fr in enumerate(field_records):
-            if fr.parent_field_id == i and fr.type_name != "":
+            if fr.parent_field_id == i and not fr.field_name.startswith("_"):
                 keys.append(fr.field_name)
         return keys
 
+    # TODO: this is still missing a lot of functionality
     def keys(
         self,
         *,
         filter_name=None,
         filter_typename=None,
+        filter_field=None,
         recursive=False,
         full_paths=True,
-        # TODO: some arguments might be missing when compared with TTree. Solve when blocker is present in dask/coffea.
+        **_,  # For compatibility reasons we just ignore other kwargs
     ):
-        if filter_name:
-            # Return keys from the filter_name list:
-            return [key for key in self._keys if key in filter_name]
-        else:
-            return self._keys
+        filter_name = uproot._util.regularize_filter(filter_name)
+        return [key for key in self._keys if filter_name(key)]
 
     @property
     def _key_indices(self):
         indices = []
         field_records = self.field_records
         for i, fr in enumerate(field_records):
-            if fr.parent_field_id == i and fr.type_name != "":
+            if fr.parent_field_id == i and not fr.field_name.startswith("_"):
                 indices.append(i)
         return indices
 
@@ -229,7 +227,7 @@ class Model_ROOT_3a3a_RNTuple(uproot.model.Model):
         d = {}
         field_records = self.field_records
         for i, fr in enumerate(field_records):
-            if fr.parent_field_id == i and fr.type_name != "":
+            if fr.parent_field_id == i and not fr.field_name.startswith("_"):
                 d[fr.field_name] = i
         return d
 
@@ -241,6 +239,11 @@ class Model_ROOT_3a3a_RNTuple(uproot.model.Model):
                 f"""memberwise serialization of {type(self).__name__}
 in file {self.file.file_path}"""
             )
+        # Probably no one will encounter this, but just in case something doesn't work correctly
+        if sys.byteorder != "little":
+            raise NotImplementedError(
+                "RNTuple reading is only supported on little-endian systems"
+            )
 
         (
             self._members["fVersionEpoch"],
@@ -282,6 +285,8 @@ in file {self.file.file_path}"""
         self._length = None
 
         self._page_list_envelopes = []
+        self._cluster_summaries = None
+        self._page_link_list = None
 
         self.ntuple = self
 
@@ -425,7 +430,19 @@ in file {self.file.file_path}"""
 
     @property
     def cluster_summaries(self):
-        return self.page_list_envelopes.cluster_summaries
+        if self._cluster_summaries is None:
+            self._cluster_summaries = []
+            for pl in self.page_list_envelopes:
+                self._cluster_summaries.extend(pl.cluster_summaries)
+        return self._cluster_summaries
+
+    @property
+    def page_link_list(self):
+        if self._page_link_list is None:
+            self._page_link_list = []
+            for pl in self.page_list_envelopes:
+                self._page_link_list.extend(pl.pagelinklist)
+        return self._page_link_list
 
     @property
     def num_entries(self):
@@ -506,8 +523,8 @@ in file {self.file.file_path}"""
                 decomp_chunk, cursor = self.read_locator(
                     loc, link.env_uncomp_size, context
                 )
-                self._page_list_envelopes = PageLink().read(
-                    decomp_chunk, cursor, context
+                self._page_list_envelopes.append(
+                    PageLink().read(decomp_chunk, cursor, context)
                 )
 
         return self._page_list_envelopes
@@ -524,6 +541,8 @@ in file {self.file.file_path}"""
         dt_str = uproot.const.rntuple_col_num_to_dtype_dict[dtype_byte]
         if dt_str == "bit":
             dt_str = "bool"
+        elif dtype_byte in uproot.const.rntuple_custom_float_types:
+            dt_str = "float32"
         return ak.forms.NumpyForm(
             dt_str,
             form_key=form_key,
@@ -546,6 +565,8 @@ in file {self.file.file_path}"""
             )
 
         rel_crs = self._column_records_dict[cfid]
+        # for this part we can use the default (zeroth) representation
+        rel_crs = [c for c in rel_crs if c.repr_idx == 0]
 
         if len(rel_crs) == 1:  # base case
             cardinality = "RNTupleCardinality" in self.field_records[field_id].type_name
@@ -662,7 +683,7 @@ in file {self.file.file_path}"""
         for i in range(len(field_records)):
             if i not in seen:
                 ff = self.field_form(i, seen)
-                if field_records[i].type_name != "":
+                if not field_records[i].field_name.startswith("_"):
                     recordlist.append(ff)
 
         form = ak.forms.RecordForm(recordlist, topnames, form_key="toplevel")
@@ -673,9 +694,14 @@ in file {self.file.file_path}"""
         context = {}
         # bool in RNTuple is always stored as bits
         isbit = dtype_str == "bit"
-        len_divider = 8 if isbit else 1
         num_elements = len(destination)
-        num_elements_toread = int(numpy.ceil(num_elements / len_divider))
+        if isbit:
+            num_elements_toread = int(numpy.ceil(num_elements / 8))
+        elif dtype_str in ("real32trunc", "real32quant"):
+            num_elements_toread = int(numpy.ceil((num_elements * 4 * nbits) / 32))
+            dtype = numpy.dtype("uint8")
+        else:
+            num_elements_toread = num_elements
         uncomp_size = num_elements_toread * dtype.itemsize
         decomp_chunk, cursor = self.read_locator(loc, uncomp_size, context)
         content = cursor.array(
@@ -722,6 +748,23 @@ in file {self.file.file_path}"""
                 .reshape(-1, 8)[:, ::-1]
                 .reshape(-1)
             )
+        elif dtype_str in ("real32trunc", "real32quant"):
+            if nbits == 32:
+                content = content.view(numpy.uint32)
+            elif nbits % 8 == 0:
+                new_content = numpy.zeros((num_elements, 4), numpy.uint8)
+                nbytes = nbits // 8
+                new_content[:, :nbytes] = content.reshape(-1, nbytes)
+                content = new_content.view(numpy.uint32).reshape(-1)
+            else:
+                ak = uproot.extras.awkward()
+                vm = ak.forth.ForthMachine32(
+                    f"""input x output y uint32 {num_elements} x #{nbits}bit-> y"""
+                )
+                vm.run({"x": content})
+                content = vm["y"]
+            if dtype_str == "real32trunc":
+                content <<= 32 - nbits
 
         # needed to chop off extra bits incase we used `unpackbits`
         destination[:] = content[:num_elements]
@@ -753,7 +796,11 @@ in file {self.file.file_path}"""
         return res
 
     def read_col_page(self, ncol, cluster_i):
-        linklist = self.page_list_envelopes.pagelinklist[cluster_i]
+        linklist = self.ntuple.page_link_list[cluster_i]
+        # Check if the column is suppressed and pick the non-suppressed one if so
+        if ncol < len(linklist) and linklist[ncol].suppressed:
+            rel_crs = self._column_records_dict[self.column_records[ncol].field_id]
+            ncol = next(cr.idx for cr in rel_crs if not linklist[cr.idx].suppressed)
         pagelist = linklist[ncol].pages if ncol < len(linklist) else []
         dtype_byte = self.column_records[ncol].type
         dtype_str = uproot.const.rntuple_col_num_to_dtype_dict[dtype_byte]
@@ -762,6 +809,8 @@ in file {self.file.file_path}"""
             dtype = numpy.dtype([("index", "int64"), ("tag", "int32")])
         elif dtype_str == "bit":
             dtype = numpy.dtype("bool")
+        elif dtype_byte in uproot.const.rntuple_custom_float_types:
+            dtype = numpy.dtype("uint32")  # for easier bit manipulation
         else:
             dtype = numpy.dtype(dtype_str)
         res = numpy.empty(total_len, dtype)
@@ -769,7 +818,11 @@ in file {self.file.file_path}"""
         zigzag = dtype_byte in uproot.const.rntuple_zigzag_types
         delta = dtype_byte in uproot.const.rntuple_delta_types
         index = dtype_byte in uproot.const.rntuple_index_types
-        nbits = uproot.const.rntuple_col_num_to_size_dict[dtype_byte]
+        nbits = (
+            self.column_records[ncol].nbits
+            if ncol < len(self.column_records)
+            else uproot.const.rntuple_col_num_to_size_dict[dtype_byte]
+        )
         tracker = 0
         cumsum = 0
         for page_desc in pagelist:
@@ -789,6 +842,15 @@ in file {self.file.file_path}"""
             res = _from_zigzag(res)
         elif delta:
             res = numpy.cumsum(res)
+        elif dtype_str == "real32trunc":
+            res = res.view(numpy.float32)
+        elif dtype_str == "real32quant" and ncol < len(self.column_records):
+            min_value = self.column_records[ncol].min_value
+            max_value = self.column_records[ncol].max_value
+            res = min_value + res.astype(numpy.float32) * (max_value - min_value) / (
+                (1 << nbits) - 1
+            )
+            res = res.astype(numpy.float32)
         return res
 
     def arrays(
@@ -1152,9 +1214,6 @@ class RNTupleSchemaExtension:
 class FooterReader:
     def __init__(self):
         self.extension_header_links = RNTupleSchemaExtension()
-        self.cluster_summary_frames = ListFrameReader(
-            RecordFrameReader(ClusterSummaryReader())
-        )
         self.cluster_group_record_frames = ListFrameReader(
             RecordFrameReader(ClusterGroupRecordReader())
         )
@@ -1189,15 +1248,25 @@ class RNTupleField:
                 continue
             if (
                 fr.parent_field_id == self.index
-                and fr.type_name != ""
                 and not fr.field_name.startswith("_")
                 and not fr.field_name.startswith(":_")
             ):
                 keys.append(fr.field_name)
         return keys
 
-    def keys(self):
-        return self._keys
+    # TODO: this is still missing a lot of functionality
+    def keys(
+        self,
+        *,
+        filter_name=None,
+        filter_typename=None,
+        filter_field=None,
+        recursive=False,
+        full_paths=True,
+        **_,  # For compatibility reasons we just ignore other kwargs
+    ):
+        filter_name = uproot._util.regularize_filter(filter_name)
+        return [key for key in self._keys if filter_name(key)]
 
     @property
     def name(self):
@@ -1222,7 +1291,7 @@ class RNTupleField:
         indices = []
         field_records = self.ntuple.field_records
         for i, fr in enumerate(field_records):
-            if fr.parent_field_id == self.index and fr.type_name != "":
+            if fr.parent_field_id == self.index and not fr.field_name.startswith("_"):
                 indices.append(i)
         return indices
 
@@ -1231,7 +1300,7 @@ class RNTupleField:
         d = {}
         field_records = self.ntuple.field_records
         for i, fr in enumerate(field_records):
-            if fr.parent_field_id == self.index and fr.type_name != "":
+            if fr.parent_field_id == self.index and not fr.field_name.startswith("_"):
                 d[fr.field_name] = i
         return d
 
